local BossLogger = CreateFrame("Frame","BossLoggerFrame")

BossLogger:SetScript("OnEvent", 
function(self, event, ...)
	return self[event](...)
end)

-- v2
local defaults = {
	["verbose"] = true,
	["chatlog"] = true,
	["transcriptor"] = true,
	["party"] = true,
	["raid"] = true,
}

BossLogger:RegisterEvent("PLAYER_TARGET_CHANGED") -- conditional startup
BossLogger:RegisterEvent("UPDATE_MOUSEOVER_UNIT") -- conditional startup
BossLogger:RegisterEvent("UNIT_COMBAT") -- conditional for end if started or startup if not
BossLogger:RegisterEvent("PLAYER_REGEN_ENABLED") -- end condition
BossLogger:RegisterEvent("PLAYER_REGEN_DISABLED") -- combat detect
BossLogger:RegisterEvent("ZONE_CHANGED") -- end condition
BossLogger:RegisterEvent("ZONE_CHANGED_NEW_AREA") -- end condition
BossLogger:RegisterEvent("ADDON_LOADED") -- Saved Variables

local transcriptor_loaded
local BLVersion = GetAddOnMetadata("BossLogger", "Version")
local isLogging, canstop, boss, inRaid, inParty = nil, nil, nil, nil, nil
local UnitLevel = UnitLevel
local UnitClassification = UnitClassification
local UnitExists = UnitExists
local UnitIsCorpse = UnitIsCorpse
local UnitIsDead = UnitIsDead
local UnitIsEnemy = UnitIsEnemy
local UnitPlayerControlled = UnitPlayerControlled
local UnitIsPlayer = UnitIsPlayer
local UnitAffectingCombat = UnitAffectingCombat
local IsInRaid = IsInRaid
local IsInGroup = IsInGroup

local playerlevel = UnitLevel("player")
local BLabel = "|cffFF9900BL:|r"
BossLogger.BossID = LibStub("LibBossIDs-1.0").BossIDs

local function Transcriptor_Loaded()
	transcriptor_loaded = Transcriptor and Transcriptor.StartLog and Transcriptor.StopLog
	return transcriptor_loaded
end

local function BossLogger_ChatPrint(msg, force)
 	if force or BossLoggerDB.verbose then
		if DEFAULT_CHAT_FRAME then
			DEFAULT_CHAT_FRAME:AddMessage(BLabel..msg);
		end
	end
end

local function BL_Slasher(cmd)
	if cmd=="" or cmd=="?" or cmd=="help" then
		BossLogger_ChatPrint("/blg verbose toggles chatframe notifications", true)
		BossLogger_ChatPrint("/blg chatlog toggles logging chat", true)
		BossLogger_ChatPrint("/blg transcriptor Transcriptor logging (if installed)", true)
		BossLogger_ChatPrint("/blg party toggles logging 5man", true)
		BossLogger_ChatPrint("/blg raid toggles logging raids", true)
	elseif strlower(cmd) == "verbose" then
		if BossLoggerDB.verbose then
			BossLoggerDB.verbose = nil
			BossLogger_ChatPrint("verbose: OFF",true)
		else
			BossLoggerDB.verbose = true
			BossLogger_ChatPrint("verbose: ON",true)
		end
	elseif strlower(cmd) == "chatlog" then
		if BossLoggerDB.chatlog then
			BossLoggerDB.chatlog = nil
			BossLogger_ChatPrint("chatlog: OFF",true)
		else
			BossLoggerDB.chatlog = true
			BossLogger_ChatPrint("chatlog: ON",true)
		end
	elseif strlower(cmd) == "transcriptor" then
		if BossLoggerDB.transcriptor then
			BossLoggerDB.transcriptor = nil
			BossLogger_ChatPrint("transcriptor: OFF",true)
		else
			BossLoggerDB.transcriptor = true
			BossLogger_ChatPrint("transcriptor: ON",true)
		end
	elseif strlower(cmd) == "party" then
		if BossLoggerDB.party then
			BossLoggerDB.party = nil
			BossLogger_ChatPrint("party: OFF",true)
		else
			BossLoggerDB.party = true
			BossLogger_ChatPrint("party: ON",true)
		end
	elseif strlower(cmd) == "raid" then
		if BossLoggerDB.raid then
			BossLoggerDB.raid = nil
			BossLogger_ChatPrint("raid: OFF",true)
		else
			BossLoggerDB.raid = true
			BossLogger_ChatPrint("raid: ON",true)
		end
	end
end

local function BossLogger_Log(flag, messg, unit)
	if flag then
		LoggingCombat(flag);
		isLogging = flag;
		boss = UnitName(unit);
		if messg then
			BossLogger_ChatPrint(messg..boss.."|r\'"); 
		end

		if BossLoggerDB.chatlog then
			LoggingChat(flag);
		end
		if BossLoggerDB.transcriptor and (transcriptor_loaded or Transcriptor_Loaded()) then
			Transcriptor:StartLog(not BossLoggerDB.verbose)
		end
		
	else
		LoggingCombat(flag);
		isLogging = nil;
		canstop = nil;
		boss = nil;
		if messg then
			BossLogger_ChatPrint(messg);
		end

		if BossLoggerDB.chatlog then
			LoggingChat(flag);
		end
		if BossLoggerDB.transcriptor and (transcriptor_loaded or Transcriptor_Loaded()) then
			Transcriptor:StopLog(not BossLoggerDB.verbose)
		end
		
	end
end

local function BossLogger_CheckLogging(unit)
	inRaid = IsInRaid()
	inParty = (not inRaid) and IsInGroup()
	-- trivial cases
	if not unit or not UnitExists(unit)
	or UnitIsCorpse(unit) 
	or UnitIsDead(unit) 
	or UnitPlayerControlled(unit) 
	or UnitIsPlayer(unit) 
	or (not inRaid and not inParty) then
	 return
	end
	-- options
	if inRaid and not BossLoggerDB.raid then
		return
	end
	if inParty and not BossLoggerDB.party then
		return
	end
	
	local guid
	local unitguid = UnitGUID(unit)
	if unitguid then guid = tonumber(unitguid:sub(6,10),16) end
	guid = guid or -1
	if UnitClassification(unit) == "worldboss" 
	or (playerlevel == MAX_PLAYER_LEVEL and UnitLevel(unit) == -1) 
	or (BossLogger.BossID[guid]) 
	or (BossLogger_ExtraIDs[guid]) then
		boss = UnitName(unit)
		if not isLogging then
			BossLogger_Log(true, "Logging \'|cffFF9900", unit);
		end
	end
end

function BossLogger.PLAYER_TARGET_CHANGED(...)
 		if isLogging and inCombat then return end
		BossLogger_CheckLogging("target")
		BossLogger_CheckLogging("targettarget") -- let's see if this makes it a bit more healer friendly.
end

function BossLogger.UPDATE_MOUSEOVER_UNIT(...)
 		if isLogging and inCombat then return end
		BossLogger_CheckLogging("mouseover")
end

function BossLogger.UNIT_COMBAT(...)
	if canstop then return end
	local unit = ...
	if isLogging then 
		if boss and unit and (UnitName(unit) == boss) then
			canstop = true;
			BossLogger_ChatPrint("\'|cffFF9900"..boss.."|r\' detected.");
		end
	else
		BossLogger_CheckLogging(unit)
	end
end

function BossLogger.PLAYER_REGEN_DISABLED() -- ic
	inCombat = true
end

function BossLogger.PLAYER_REGEN_ENABLED() -- ooc
	if isLogging and canstop then
		if not (UnitIsDead("player") or UnitIsCorpse("player") or UnitIsGhost("player")) then
			BossLogger_Log(false,"Logging stopped \'|cffFF9900"..boss.."|r\' (combat end)");
		end
	end
	inCombat = nil
end

function BossLogger.ZONE_CHANGED(...)
	if isLogging and not inCombat then -- see if this helps with encounters that include zone changes (eg. gunship)
		BossLogger_Log(false, "Logging stopped \'|cffFF9900"..boss.."|r\' (zoning)");
	end
end

function BossLogger.ZONE_CHANGED_NEW_AREA(...)
	if isLogging then
		BossLogger_Log(false, "Logging stopped \'|cffFF9900"..boss.."|r\' (zoning)");
	end
end

function BossLogger.ADDON_LOADED(...)
	local addon = ...
	if addon == "BossLogger" then
		BossLoggerDB = BossLoggerDB or CopyTable(defaults)
		if (not BossLoggerDB["version"]) or (BossLoggerDB["version"] < BLVersion) then -- bump major version of you change SV
			-- upgrade SVs if needed
			BossLoggerDB = CopyTable(defaults)
			BossLoggerDB["version"] = BLVersion
		end
		SlashCmdList["BOSSLOGGER"] = BL_Slasher
		SLASH_BOSSLOGGER1 = "/bosslogger"
		SLASH_BOSSLOGGER2 = "/blg"
		BossLogger:UnregisterEvent("ADDON_LOADED")
	end
end

--[[
Transcriptor:StartLog(silent)
Transcriptor:StopLog(silent)
]]