/**
 * EpicMusicPlayer Playlistgenerator
 *
 * @author yess
 */

package app;

import com.ice.jni.registry.Registry;
import com.ice.jni.registry.RegistryKey;
import gui.AboutBox;
import gui.SelectAccountDialog;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JOptionPane;

/**
 *
 * 
 */
public class Config {

    private File fMusicDir;
    private File fConfigFileName;
    private File fWowDirectory;
    private String loadfrom = "";
    private String saveto = "";
    
    private boolean duplicate = false;
    private boolean saved = true;
    private boolean showWarning = true;
    private boolean addGameMusicMode = false;
    private boolean firstRun = true;
    private boolean showWelcomeDialog = true;
    private String lastSelectedFolder = " ";
    private boolean runningAsAdmin = false;
    
    private String savedVersion = "";
    private boolean copyfiles = true;
    private Junction jLink;
    private String cantAccsess = "";

    private String playlistVersion = "3.1";
    
    public Config(String fileName) throws FileNotFoundException, IOException, Exception {
        fConfigFileName = new File(fileName);
        
        if(fConfigFileName.exists()){
            //load config
            this.load(fConfigFileName.getPath());
        }
        else //new configuration
        {
            //try read path from registry
            fWowDirectory = this.getWoWFolderRegistry();
            init(fWowDirectory);
        }  
     }
    
     public Config(String fileName, File wowDir) throws FileNotFoundException, Exception{
         fConfigFileName = new File(fileName);
         init(wowDir);
     }
    
    private void init(File wowfolder) throws FileNotFoundException, Exception{       
        //try set wow folder
        setWowfolder(wowfolder);
        /*
        wow folder is fine create music folder
        File musicdir = createDefaultMusicDir(getWoWDir());
        setMusicDir(musicdir);
        //get account(s) and promt select if more then one
        setLoadfrom(selectAccount((ArrayList) getAccunts(getWoWDir())));
        setSaveto("all-accounts");                    
        */
    }
    
    /**
     * Check all needed files/folder for write access.
     * @return false if one file/folder is protected.
     */
    private boolean checkAccess(){
        if(fConfigFileName.canWrite()){
            cantAccsess = fConfigFileName.getPath();
            return false;
        }
        if(!fWowDirectory.canWrite()){
            cantAccsess = fWowDirectory.getPath();
            return false;
        }
        return true;
    } 
    
    public void load(String filename) throws FileNotFoundException, IOException, Exception {
        BufferedReader in = new BufferedReader(new FileReader(filename));
        String line = in.readLine();
        String split[];

        while (line != null) {
            if (line.contains("musicfolder")) {
                split = line.split("=");
                setMusicFloder(new File(split[1]));
            }

            if (line.contains("wowfolder")) {
                split = line.split("=");
                setWoWDir(new File(split[1]));
            }
            if (line.contains("lastSelectedFolder")) {
                split = line.split("=");
                setLastSelectedFolder(split[1]);
            }
            if (line.contains("loadfrom")) {
                split = line.split("=");
                setLoadfrom(split[1]);
            }
            if (line.contains("saveto")) {
                split = line.split("=");
                setSaveto(split[1]);
            }
            if (line.contains("dublicate")) {
                split = line.split("=");
                if (split[1].equalsIgnoreCase("true")) {
                    setDuplicate(true);
                }
            }
            if (line.contains("copyfiles")) {
                split = line.split("=");
                if (split[1].equalsIgnoreCase("false")) {
                    setCopyfiles(false);
                }
            }
            if (line.contains("showWarning")) {
                split = line.split("=");
                if (split[1].equalsIgnoreCase("false")) {
                    setShowWarning(false);
                }
            }
            if (line.contains("showWelcomeDialog")) {
                split = line.split("=");
                if (split[1].equalsIgnoreCase("false")) {
                    setShowWelcomeDialog(false);
                }
            }
            if (line.contains("addGameMusicMode")) {
                split = line.split("=");
                if (split[1].equalsIgnoreCase("true")) {
                    setAddGameMusicMode(true);
                    System.out.println("addGameMusicMode=true");
                } 
            }
            if (line.contains("version")) {
                split = line.split("=");
                setSavedVersion(split[1]);
            }
            line = in.readLine();
            firstRun = false;
        }
    }
    
    public void save() throws IOException {
        BufferedWriter out = new BufferedWriter(new FileWriter(fConfigFileName.getPath(), false));
        out.write("musicfolder=" + this.fMusicDir);
        out.newLine();
        out.write("wowfolder=" + getWoWDir());
        out.newLine();
        out.write("lastSelectedFolder=" + getLastSelectedFolder());
        out.newLine();
        out.write("loadfrom=" + getLoadfrom());
        out.newLine();
        out.write("saveto=" + getSaveto());
        out.newLine();
        out.write("dublicate=" + isDuplicate());
        out.newLine();
        out.write("copyfiles=" + isCopyfiles());
        out.newLine();
        out.write("showWarning=" + isShowWarning());
        out.newLine();
        out.write("showWelcomeDialog=" + isShowWelcomeDialog());
        out.newLine();
        out.write("version=" + getVersion());
        out.newLine();
        out.write("addGameMusicMode=" + isAddGameMusicMode());
        out.newLine();
        
        out.close();
    }
    
    public String getVersion()
    {
        org.jdesktop.application.ResourceMap resourceMap = org.jdesktop.application.Application.getInstance(app.EMPApp.class).getContext().getResourceMap(AboutBox.class);
        return resourceMap.getString("Application.version");
    }
    
    public boolean isUpdate()
    {
        if(!getSavedVersion().equals("") && getVersion().compareTo(getSavedVersion()) > 0)
        {
            System.out.print(getVersion() +" > "+ this.getSavedVersion());
            return true;
        }
        return false;
    }

    public void setWowfolder(File wowfolder) throws FileNotFoundException {
        //check if in wow folder
        File addondir = findWowDir(wowfolder);
        if(addondir==null){
            throw new FileNotFoundException("wow.exe not found in selected World of Warcraft folder: "+wowfolder.getPath());
        }
        else
        {
            this.fWowDirectory = addondir;
        }   
    }

    /**
     * Set music folder without check if in wow folder
     * @param file
     */
    private void setMusicFloder(File file) {
        if(file == null)
            throw new IllegalArgumentException("MusicFloder not set.");
        if(file.getPath().equals("") || file.getPath().equalsIgnoreCase("null"))
            throw new IllegalArgumentException("MusicFloder not set.");
        
        this.fMusicDir = file;
    }

    private void setWoWDir(File file)
    {
        if(file == null)
            throw new IllegalArgumentException("WoW folder not set.");
        if(file.getPath().equals("") || file.getPath().equalsIgnoreCase("null"))
            throw new IllegalArgumentException("WoW folder not set.");
        
        fWowDirectory = file;
    }
    
    public File getWoWDir()
    {
        return fWowDirectory;
    }
    
    public File getWoWFolderRegistry() throws RegistryException
    {
        File wowfolder = null;
        //try read path from registry
        try {
            //Registry reg = new Registry();
            RegistryKey topKey = Registry.getTopLevelKey("HKEY_LOCAL_MACHINE");
            RegistryKey subKey = Registry.openSubkey(topKey, "SOFTWARE\\Blizzard Entertainment\\World of Warcraft", 0);
            //System.out.println(subKey.getStringValue("InstallPath"));
            //wowfolder = new File(subKey.getStringValue("InstallPath"));
            String gamepath = subKey.getStringValue("GamePath");
            wowfolder = new File(gamepath.replace("Launcher.exe", ""));
        } catch (UnsatisfiedLinkError linkerror) {
            Logger.getLogger("epicmusicplayer").log(Level.FINEST, linkerror.getMessage(), linkerror);
            Logger.getLogger("epicmusicplayer").log(Level.INFO,"Could not read the WoW install path from registry.");
            throw new RegistryException("Could not read the WoW install path from registry.");
        } catch (Exception e) {
            Logger.getLogger("epicmusicplayer").log(Level.FINEST, e.getMessage(), e);
            Logger.getLogger("epicmusicplayer").log(Level.INFO,"Could not read the WoW install path from registry.");
            throw new RegistryException("Could not read the WoW install path from registry.");
        }
        return wowfolder;
    }
    
    public void setMusicDir(File musicdir) throws FileNotFoundException {
        if (!musicdir.exists()) {
            throw new FileNotFoundException("Directory not found.\n" + musicdir.getPath());
        }

        //check if in wow folder
        File addondir = findWowDir(musicdir);
        if(addondir==null){
            throw new FileNotFoundException("Music folder is not in World of Warcraft folder");
        }
        else
        {
            //set music folder
            System.out.println("wowdir="+addondir.getPath());
            this.fMusicDir = musicdir;
            this.fWowDirectory = addondir;
        }   
        
    }
    
    private File findWowDir(File musicdir)
    {
        
        while(musicdir !=null)
        {    
            System.out.println("findWowDir: cheking="+musicdir.getPath());
            Logger.getLogger("epicmusicplayer").log(Level.FINEST, "findWowDir: cheking="+musicdir.getPath());
            String[] children = musicdir.list();
            for(int i=0; i<children.length;i++)
            {
                if(children[i].equalsIgnoreCase("WTF"))
                {
                    return musicdir; //wow folder found
                }
            }
            musicdir = musicdir.getParentFile();
        }
        return null;
    }
    
    public File getPlaylistFile(String accountname){
        return new File(getWoWDir().getPath() + "/WTF/Account/" + accountname + "/SavedVariables/EpicMusicPlayer.lua");
    }

    public List getAccunts(File wowdir) throws FileNotFoundException {

        File accountsdir = new File(wowdir.getPath() + "/WTF/Account/");

        if (!accountsdir.exists()) {
            throw new FileNotFoundException("Account directory not found in: \n" + accountsdir.getPath());
        }

        ArrayList list = new ArrayList();
        String[] children = accountsdir.list();
        for (int i = 0; i < children.length; i++) {
            //System.out.println(children[i]);
            if(!children[i].startsWith(".")){
                list.add(children[i]);
            }
        }


        return list;
    }
    
    public File getMusicDir() {
        return fMusicDir;
    }

    public boolean isSaved() {
        return saved;
    }

    public void setSaved(boolean saved) {
        this.saved = saved;
    }

    

    public String getLoadfrom() {
        return loadfrom;
    }

    public void setLoadfrom(String loadfrom) {
        if(loadfrom == null)
            throw new IllegalArgumentException("Account not set.");
        if(loadfrom.equals("") || loadfrom.equalsIgnoreCase("null"))
            throw new IllegalArgumentException("Account not set.");
        
        this.loadfrom = loadfrom;
    }

    public String getSaveto() {
        return saveto;
    }

    public void setSaveto(String saveto) {
        if(saveto == null)
            throw new IllegalArgumentException("Account not set.");
        if(saveto.equals("") || saveto.equalsIgnoreCase("null"))
            throw new IllegalArgumentException("Account not set.");
        
        this.saveto = saveto;
    }

    public boolean isDuplicate() {
        return duplicate;
    }

    public void setDuplicate(boolean duplicate) {
        this.duplicate = duplicate;
    }

    public boolean isCopyfiles() {
        return copyfiles;
    }

    public void setCopyfiles(boolean copyfiles) throws Exception {
        if(!copyfiles){
            this.setJLink(new Junction());
        }
        this.copyfiles = copyfiles;
    }

    public Junction getJLink() {
        return jLink;
    }

    public void setJLink(Junction jLink) {
        this.jLink = jLink;
    }

    public boolean isShowWarning() {
        return showWarning;
    }

    public void setShowWarning(boolean showWarning) {
        this.showWarning = showWarning;
    }

    public String getSavedVersion() {
        return savedVersion;
    }

    public void setSavedVersion(String savedVersion) {
        this.savedVersion = savedVersion;
    }

    public boolean isAddGameMusicMode() {
        return addGameMusicMode;
    }

    public void setAddGameMusicMode(boolean addGameMusicMode) {
        this.addGameMusicMode = addGameMusicMode;
    }

    public boolean isFirstRun() {
        return firstRun;
    }

    public void setFirstRun(boolean firstRun) {
        this.firstRun = firstRun;
    }

    public boolean isShowWelcomeDialog() {
        return showWelcomeDialog;
    }

    public void setShowWelcomeDialog(boolean showWelcomeDialog) {
        this.showWelcomeDialog = showWelcomeDialog;
    }

    public String getLastSelectedFolder() {
        return lastSelectedFolder;
    }

    public void setLastSelectedFolder(String lastSelectedFolder) {
        this.lastSelectedFolder = lastSelectedFolder;
    }

    public boolean isRunningAsAdmin() {
        return runningAsAdmin;
    }

    public void setRunningAsAdmin(boolean runningAsAdmin) {
        this.runningAsAdmin = runningAsAdmin;
    }

    public String getPlaylistVersion() {
        return playlistVersion;
    }

}
