
package app;

import gui.*;
import java.io.BufferedInputStream;
import playlist.Song;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.util.ArrayList;
import java.util.logging.Level;
import java.util.logging.Logger;
import javazoom.jl.decoder.JavaLayerException;
import javazoom.jlgui.basicplayer.BasicController;
import javazoom.jlgui.basicplayer.BasicPlayer;
import javazoom.jlgui.basicplayer.BasicPlayerException;
import playlist.Playlists;

import javazoom.jl.player.AudioDevice;
import javazoom.jl.player.FactoryRegistry;
import javazoom.jl.player.Player;

public class EMPPlayer implements Runnable {

    private BasicController control;
    
    private ArrayList<Song> songlist;
    private int mp3index;
    private int listindex;
    private Config conf;
    private MainView mainFrame;
    private Playlists playList;
    
    private boolean play;
    private Player player;
    
    public EMPPlayer(Playlists playlist, Config conf, MainView f) throws FileNotFoundException, JavaLayerException {
        this.playList = playlist;
        this.conf = conf;
        this.mainFrame = f;
        BasicPlayer player2 = new BasicPlayer();
        control = (BasicController) player2;
    }

    public void stop() {
        
        //kill running thread
        play = false;
        if (player != null) {
            player.close();
        }
        
        if (control != null) {
            try {
                control.stop();
                mainFrame.setSong("", "");
            } catch (BasicPlayerException ex) {
                Logger.getLogger(EMPPlayer.class.getName()).log(Level.SEVERE, null, ex);
            }
        }
    }

    public void play(int listindex, int mp3index) throws FileNotFoundException, BasicPlayerException, JavaLayerException {
        if(mp3index < 1)
        {
            Logger.getLogger(EMPPlayer.class.getName()).log(Level.INFO, "Not playing index < 1 mp3index="+mp3index);
            return;
        }
        
        this.mp3index = mp3index;
        this.listindex = listindex;
        
        //kill running thread
        play = false;
        if (player != null) {
            player.close();
        }
        
        
        songlist = playList.getPlaylist(listindex);
        Song s= songlist.get(mp3index);
        
        String path = s.getPath();
        path = path.replaceAll("\\\\\\\\", "/");
        String album = s.getAlbum();
        path = conf.getMusicDir().getPath() + "/"+path;

        if(album != null && (album.equals("ingame") || s.getInGame()) && !conf.isAddGameMusicMode())
        {
            mainFrame.log("Can't play ingame songs, sorry.");
            return;
        }
        
        if(path.toUpperCase().endsWith(".WAV"))
        {
            mainFrame.log("Can't play wav songs, sorry.");
            return;
        }
        
        if(path.toUpperCase().endsWith(".MP3"))
        {
            FileInputStream fin = new FileInputStream(path);
            BufferedInputStream bin = new BufferedInputStream(fin);

            AudioDevice dev = FactoryRegistry.systemRegistry().createAudioDevice();
            player = new Player(bin, dev);
            play = true;
            //System.out.println("playing "+listindex+"  "+mp3index);
            Logger.getLogger("epicmusicplayer").log(Level.INFO, "playing "+path);
            //Logger.getLogger("app.EMPApp").log(Level.INFO, "playing "+path);
            Thread thplayer = new Thread(this);
            thplayer.start();
        }
        else //use javazoom.jlgui.basicplayer.BasicPlayer
        {
            // Open file, or URL or Stream (shoutcast) to play.
            control.open(new File(path));
            // control.open(new URL("http://yourshoutcastserver.com:8000"));

            // Start playback in a thread.
            control.play();
        }
        
        mainFrame.setSong(playList.getPlaylistName(listindex)+": ","["+s.getArtist()+" "+s.getTitle()+"]");
        Logger.getLogger("epicmusicplayer").log(Level.INFO, "playing "+path);
        
    }
    
    public void playnext()
    {
        boolean again = true; //play next file on error
        
        while(again)
        {
        //System.out.println("playnext "+ listindex+"  "+mp3index+"  mp3list.size()"+songlist.size());
        try {
            if(mp3index < songlist.size()-1)
            {
                mp3index++;
            }
            else if(listindex < playList.getPlaylistCount()-1)
            {
                //end of list play first song on next list
                listindex++;
                mp3index = 1; //first song(0) is playlistname!
            }
            else //no more lists play first list
            {
                listindex = 0;
                mp3index = 1;
            }
            
            //finaly play something^^
            play(listindex, mp3index);
            again = false;
            
            
        } catch (JavaLayerException ex) {
            Logger.getLogger("epicmusicplayer").log(Level.SEVERE, null, ex);
            again = true; //play next file on error
        } catch (FileNotFoundException ex) {
            Logger.getLogger("epicmusicplayer").log(Level.SEVERE, null, ex);
            again = true; //play next file on error
        } catch (BasicPlayerException ex) {
            Logger.getLogger("epicmusicplayer").log(Level.SEVERE, null, ex);
            again = true; //play next file on error
        }
        
        }
    }

    
    public void playlast()
    {
        boolean again = true; //play next file on error
        
        while(again)
        {
        //System.out.println("playnext "+ listindex+"  "+mp3index+"  mp3list.size()"+songlist.size());
        try {
            if(mp3index > 0) //first song(0) is playlistname!
            {
                mp3index--;
            }
            else 
            {
                mp3index = 1;
            }
            
            //finaly play something^^
            play(listindex, mp3index);
            again = false;
            
            
        } catch (JavaLayerException ex) {
            Logger.getLogger("epicmusicplayer").log(Level.SEVERE, null, ex);
            again = true; //play next file on error
        } catch (FileNotFoundException ex) {
            Logger.getLogger("epicmusicplayer").log(Level.SEVERE, null, ex);
            again = true; //play next file on error
        } catch (BasicPlayerException ex) {
            Logger.getLogger("epicmusicplayer").log(Level.SEVERE, null, ex);
             again = true; //play next file on error
        }
        
        }
    }
    
    public void run() {
        //while (play) {
            try {
                player.play();
            } catch (JavaLayerException ex) {
                Logger.getLogger("epicmusicplayer").log(Level.SEVERE, null, ex);
            }
            
            if(play)
            {
                playnext(); 
            }
            //playnext();
            //play = false;
            //System.out.println("next");
        //}
    }

}
